@@ -22,6 +22,7 @@ from optparse import OptionParser, make_option
 import stgit.commands.common
 from stgit.commands.common import *
 from stgit.utils import *
+from stgit.out import *
 from stgit import stack, git
 
 
@@ -33,6 +34,7 @@ range. The applied patches are prefixed with a '+', the unapplied ones
 with a '-' and the hidden ones with a '!'. The current patch is
 prefixed with a '>'. Empty patches are prefixed with a '0'."""
 
+directory = DirectoryHasRepository()
 options = [make_option('-b', '--branch',
                        help = 'use BRANCH instead of the default one'),
            make_option('-a', '--all',
@@ -98,9 +100,9 @@ def __print_patch(patch, branch_str, prefix, empty_prefix, length, options):
         patch_str = patch_str.ljust(length)
 
     if options.description:
-        out.stdout(prefix + patch_str + ' | ' + __get_description(patch))
+        out.stdout(prefix + patch_str + ' # ' + __get_description(patch))
     elif options.author:
-        out.stdout(prefix + patch_str + ' | ' + __get_author(patch))
+        out.stdout(prefix + patch_str + ' # ' + __get_author(patch))
     else:
         out.stdout(prefix + patch_str)
 
@@ -192,17 +194,17 @@ def func(parser, options, args):
         if options.missing:
             raise CmdException, '--graphical not supported with --missing'
 
+        gitk_args = []
         if applied:
-            gitk_args = ' %s^..%s' % (git_id(applied[0]), git_id(applied[-1]))
-        else:
-            gitk_args = ''
-
+            gitk_args.append('%s^..%s'
+                             % (git_id(crt_series, applied[0]),
+                                git_id(crt_series, applied[-1])))
         for p in unapplied:
-            patch_id = git_id(p)
-            gitk_args += ' %s^..%s' % (patch_id, patch_id)
+            patch_id = git_id(crt_series, p)
+            gitk_args.append('%s^..%s' % (patch_id, patch_id))
 
-        if os.system('gitk%s' % gitk_args) != 0:
-            raise CmdException, 'gitk execution failed'
+        # discard the exit codes generated by SIGINT, SIGKILL, SIGTERM
+        Run('gitk', *gitk_args).returns([0, -2, -9, -15]).run()
     else:
         max_len = 0
         if len(patches) > 0:
