@@ -40,10 +40,10 @@ test_expect_success \
 	stg pop --all
 	'
 
-test_expect_failure \
+test_expect_success \
 	'Push the second patch with conflict' \
 	'
-	stg push bar
+	! stg push bar
 	'
 
 test_expect_success \
@@ -52,4 +52,19 @@ test_expect_success \
 	stg push --undo
 	'
 
+test_expect_success \
+	'Check the push after undo fails as well' \
+	'
+	! stg push bar
+	'
+
+test_expect_success \
+	'Undo with disappeared newborn' \
+	'
+	touch newfile &&
+	stg add newfile &&
+	rm newfile &&
+	stg push --undo
+	'
+
 test_done
